/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KQTIINTERACTION_H
#define KQTIINTERACTION_H

#include "kqtibodyelement.h"

#include <kqti_export.h>
#include <QDomElement>

/**
 * \class KQtiInteraction
 * Interactions allow the candidate to interact with the item. Through an
 * interaction, the candidate selects or constructs a response. The candidate's
 * responses are stored in the response variables. Each interaction is
 * associated with (at least) one response variable.
 * 
 * http://www.imsglobal.org/question/qtiv2p1pd2/imsqti_infov2p1pd2.html#element10264
 */

class KQTI_EXPORT KQtiInteraction : public KQtiBodyElement
{
    public:
        
        KQtiInteraction(const QString& responseIdentifier = QString());
        KQtiInteraction(const QDomElement& element);
        ~KQtiInteraction();
        
        QString responseIdentifier() const {return mResponseIdentifier;}
        
        /**
         * element
         * @return create and return the DOM element
         */
        QDomElement element() const;
        
    protected:
        QString mResponseIdentifier;
};

#endif
