/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KQTIITEMSESSION_H
#define KQTIITEMSESSION_H

#include <kqti_export.h>

/**
 * \class KQtiItemSession
 * Maintains the session of an item.
 * 
 * Not an XML entity
 * 
 * http://www.imsglobal.org/question/qtiv2p1pd2/imsqti_infov2p1pd2.html#section10055
 */

class KQtiAssessmentItem;
class KQtiItemResult;

class KQTI_EXPORT KQtiItemSession /*This is a base class*/
{
    public:
        KQtiItemSession(KQtiAssessmentItem* assessmentItem);
        virtual ~KQtiItemSession();
        
        enum KQtiItemSessionState
        {
            Begin, ///< The state before the item is eligable for being displayed to the candidate.
            Initial,
            Interacting,
            Suspended,
            ModalFeedback,
            Closed,
            Solution,
            Review
        };
        
        void setStatus(KQtiItemSessionState sessionState) {mSessionState = sessionState;}
        KQtiItemSessionState state() const {return mSessionState;}
        void setItemResult(KQtiItemResult* itemResult) {mItemResult = itemResult;}
        KQtiItemResult* itemResult() const {return mItemResult;}
        
    protected:
        KQtiAssessmentItem* mItem;
        KQtiItemSessionState mSessionState;
        KQtiItemResult* mItemResult;
};

#endif
