/***************************************************************************
 *   Copyright (C) 2010 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#if !defined(ACTIVITYMODEL__H)

#define ACTIVITYMODEL__H

#include "core/types.h"

#include "gui/activitytree.h"

#include <QList>
#include <QAbstractItemModel>

typedef QList<Activity*> ModelActivities;

class ActivityModel : public QAbstractItemModel
{
	Q_OBJECT

	public:
		static const int sortingRole = Qt::UserRole;

	public:
		ActivityModel(QObject* parent);

	public:
		int rowCount(const QModelIndex& = QModelIndex()) const { return m_Activities.size(); }
		int columnCount(const QModelIndex& = QModelIndex()) const { return ActivityTree::lastActivityCol; }
		QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
		QVariant headerData(int section, Qt::Orientation  orientation, int role = Qt::DisplayRole) const;
		Qt::ItemFlags flags(const QModelIndex& index) const;
		bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
		QModelIndex parent(const QModelIndex&) const { return QModelIndex(); }
		QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;

		Activity* activity(const QModelIndex& index);
		const Activity* activity(const QModelIndex& index) const;

		void add(Activity* a);
		void remove(Activity* a);
		void clear();
		QModelIndex find(const Activity* activity);
		QModelIndex first();

	private:
		ModelActivities m_Activities;
};

#endif

