/***************************************************************************
 *   Copyright (C) 2010 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#include "veneer/datetime.h"

#include <QDateTime>
#include <QLocale>
#include <QDebug>

namespace Veneer
{

	QString formatDateTime(const QDateTime& d)
	{
		return QLocale().toString(d, QLocale::ShortFormat);
	}

	QString formatShortDate(const QDate& d)
	{
		return QLocale().toString(d, QLocale::ShortFormat);
	}

	QString formatLongDate(const QDate& d)
	{
		return QLocale().toString(d, QLocale::LongFormat);
	}

	QString formatTime(const QTime& t)
	{
		return QLocale().toString(t);
	}

	QDate readDate(const QString& s, bool* ok)
	{
		QDate rval;
		QList<Qt::DateFormat> formats;
		formats << Qt::TextDate << Qt::ISODate << Qt::SystemLocaleDate << Qt::SystemLocaleShortDate
				<< Qt:: SystemLocaleLongDate << Qt::DefaultLocaleShortDate << Qt::DefaultLocaleLongDate;

		foreach (Qt::DateFormat f, formats)
		{
			rval = QDate::fromString(s, f);

			if (rval.isValid())
				break;
		}

		if (ok != NULL)
			*ok = rval.isValid();

		return rval;
	}
}
