/***************************************************************************
 *   Copyright (C) 2010 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#include "veneer/filedialog.h"
#include "veneer/url.h"

#include <QStringList>
#include <QFileDialog>
#include <QDebug>

static QString kdeToQtFilters(const QString& kdeFilters)
{
	QStringList sl = kdeFilters.split('\n');

	QStringList result;

	foreach(const QString& s, sl)
		result.append(s.split('|')[1]);

	return result.join(";;");
}

namespace Veneer
{

	QString getOpenFileName(QWidget* parent, const QString&)
	{
		return QFileDialog::getOpenFileName(parent, QObject::tr("Open File..."));
	}

	Url getOpenUrl(QWidget* parent, const QString&)
	{
		return QUrl::fromLocalFile(getOpenFileName(parent));
	}

	QStringList getOpenFileNames(QWidget* parent, const QString&, const QString& filters)
	{
		return QFileDialog::getOpenFileNames(parent, QObject::tr("Open Files..."), QString(), kdeToQtFilters(filters));
	}

	QList<Url> getOpenUrls(QWidget* parent, const QString&, const QString& filters)
	{
		QList<Url> result;

		const QStringList fileNames = getOpenFileNames(parent, QString(), kdeToQtFilters(filters));

		foreach(const QString& s, fileNames)
			result << s;

		return result;
	}

	Url getSaveUrl(QWidget* parent, const QString&, const QString& filters, const QString& fileName)
	{
		qDebug() << fileName;
		return Url(QFileDialog::getSaveFileName(parent, QObject::tr("Save File As..."), fileName, kdeToQtFilters(filters)));
	}

}
